Type: Package
Package: thisplot
Title: Utility Functions for Plotting
Version: 0.4.3
Date: 2026-08-01
Authors@R: 
    c(
    person("Meng", "Xu", email = "mengxu98@qq.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-8300-1054")),
    person(given = "Haoliang", family = "Zhu", email = "hurry060215@gmail.com", role = c("aut"), comment = c(ORCID = "0009-0004-9766-5179"))
    )
Maintainer: Meng Xu <mengxu98@qq.com>
Description: Provides utility functions for plotting. Includes functions for color manipulation, plot customization, panel size control, data optimization for plots, and layout adjustments.
License: MIT + file LICENSE
URL: https://mengxu98.github.io/thisplot/
BugReports: https://github.com/mengxu98/thisplot/issues
Depends: R (>= 4.1.0)
Imports: cli, ComplexHeatmap, dendextend, ggforce, geomtextpath,
        ggrepel, ggnewscale, ggplot2, grDevices, grid, gtable, igraph,
        methods, patchwork, proxyC, reshape2, stats, thisutils (>=
        0.4.3), utils
Suggests: circlize, dplyr, forcats, ggalluvial, ggupset, ggVennDiagram,
        htmltools, Matrix, png, purrr, ragg, scales, stringr, testthat
        (>= 3.0.0), tidyr, venny
Config/Needs/website: mengxu98/thistemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-US
LazyData: true
NeedsCompilation: no
Packaged: 2026-08-01 09:55:35 UTC; mx
Author: Meng Xu [aut, cre] (ORCID: <https://orcid.org/0000-0002-8300-1054>),
  Haoliang Zhu [aut] (ORCID: <https://orcid.org/0009-0004-9766-5179>)
Repository: CRAN
Date/Publication: 2026-08-01 10:10:02 UTC
