# Generated by roxygen2: do not edit by hand

S3method(drop_data,default)
S3method(drop_data,ggplot)
S3method(drop_data,patchwork)
S3method(head,colors)
S3method(print,ChineseColors)
S3method(print,colors)
S3method(print,thisplot_logo)
S3method(slim_data,default)
S3method(slim_data,ggplot)
S3method(slim_data,patchwork)
export(Blend2Color)
export(BlendRGBList)
export(ChineseColors)
export(DimDataPlot)
export(GraphPlot)
export(RGBA2RGB)
export(StatPlot)
export(add_grob)
export(adjcolors)
export(adjustlayout)
export(annotate_quadrants)
export(annotation_block_fill_graphics)
export(annotation_block_graphics)
export(annotation_graphics)
export(as_grob)
export(as_gtable)
export(blendcolors)
export(build_heatmap_annotation)
export(build_patchwork)
export(clip_symmetric_range)
export(cluster_within_group2)
export(col2hex)
export(compute_velocity_on_grid)
export(drop_data)
export(extractgrobs)
export(geom_alluvial)
export(geom_alluvial_label)
export(geom_alluvial_text)
export(geom_sankey)
export(geom_sankey_bump)
export(geom_sankey_label)
export(geom_sankey_text)
export(get_chinese_palettes)
export(get_colors)
export(get_legend)
export(get_vars)
export(grid_draw)
export(heatmap_fixsize)
export(heatmap_rendersize)
export(jitter_highlighted_points)
export(make_long)
export(mestimate)
export(normalize_drawable)
export(palette_colors)
export(panel_fix)
export(panel_fix_overall)
export(patchwork_grob)
export(segements_df)
export(show_palettes)
export(simple_colors)
export(slim_data)
export(standardise)
export(theme_alluvial)
export(theme_blank)
export(theme_sankey)
export(theme_sankey_bump)
export(theme_this)
export(thisplot_logo)
export(to_percent)
export(visual_colors)
import(ggplot2)
import(thisutils)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(utils,head)
