#' @import ggplot2 thisutils
NULL

utils::globalVariables(
  c(
    ".",
    ".data",
    "..r",
    "count",
    "Freq",
    "group",
    "groupn",
    "id",
    "intersection",
    "label",
    "name",
    "next_node",
    ".plot_group",
    ".plot_label",
    ".plot_shape",
    ".plot_split",
    ".plot_x",
    ".plot_y",
    "next_x",
    "node",
    "value",
    "X",
    "Y",
    "x",
    "y"
  )
)
