% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_colors.R
\name{head.colors}
\alias{head.colors}
\title{Return the first part of a colors object}
\usage{
\method{head}{colors}(x, n = 6L, ...)
}
\arguments{
\item{x}{A colors object (data frame with color information).}

\item{n}{Number of rows to return. Default is \code{6}.}

\item{...}{Additional arguments passed to \code{\link[=head]{head()}}.}
}
\value{
A colors object with the first \code{n} rows.
}
\description{
Returns the first part of a colors object, similar to \code{\link[=head]{head()}} for data frames.
}
\examples{
head(get_colors())

head(get_colors(), n = 10)
}
