% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{build_patchwork}
\alias{build_patchwork}
\title{Build a patchwork gtable}
\usage{
build_patchwork(
  x,
  guides = "auto",
  table_rows = 18,
  table_cols = 15,
  panel_row = 10,
  panel_col = 8
)
}
\arguments{
\item{x}{A patchwork object.}

\item{guides}{How to handle guides.
Default is \code{"auto"}.}

\item{table_rows}{The number of rows in the table grid.
Default is \code{18}.}

\item{table_cols}{The number of columns in the table grid.
Default is \code{15}.}

\item{panel_row}{The row index for panels.
Default is \code{10}.}

\item{panel_col}{The column index for panels.
Default is \code{8}.}
}
\value{
A gtable object.
}
\description{
Build a gtable from a patchwork object by arranging multiple plots
according to the layout specification.
}
