% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{standardise}
\alias{standardise}
\title{Standardize data by rows}
\usage{
standardise(data)
}
\arguments{
\item{data}{A matrix or data frame to standardize.}
}
\value{
The standardized data with the same structure as input.
}
\description{
Standardize each row of a data matrix by subtracting the mean and dividing by the standard deviation.
}
