% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{RGBA2RGB}
\alias{RGBA2RGB}
\title{Convert RGBA color to RGB with background}
\usage{
RGBA2RGB(RGBA, BackGround = c(1, 1, 1))
}
\arguments{
\item{RGBA}{A list containing RGB values and alpha channel.}

\item{BackGround}{The background RGB color to composite with.
Default is \code{c(1, 1, 1)} (white).}
}
\value{
A numeric vector of RGB values.
}
\description{
Convert an RGBA (Red, Green, Blue, Alpha) color to RGB by compositing
it with a background color based on the alpha channel.
}
