% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{jitter_highlighted_points}
\alias{jitter_highlighted_points}
\title{Jitter highlighted points deterministically}
\usage{
jitter_highlighted_points(
  data,
  jitter_width = 0.2,
  jitter_height = 0.2,
  seed = 11
)
}
\arguments{
\item{data}{A data frame containing `x`, `y`, and `border` columns.}

\item{jitter_width}{Width of the x jitter.}

\item{jitter_height}{Height of the y jitter.}

\item{seed}{Numeric value used to offset the deterministic sequence.}
}
\value{
A data frame with `x_plot` and `y_plot` columns.
}
\description{
Jitter highlighted points deterministically
}
\examples{
df <- data.frame(
  x = c(0, 1),
  y = c(0, 1),
  border = c(FALSE, TRUE)
)
jitter_highlighted_points(df, jitter_width = 0.2, jitter_height = 0.2)
}
