% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{palette_list}
\alias{palette_list}
\title{A list of palettes for use in data visualization}
\description{
A list of palettes for use in data visualization
}
\concept{data}
